﻿using System;
using System.Windows.Input;
using System.Windows.Threading;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using GalaSoft.MvvmLight.Messaging;

namespace MvvmChrono.ViewModel
{
    /// <summary>
    /// Main page <see cref="ViewModel"/>
    /// </summary>
    public class MainViewModel : ViewModelBase
    {

        /// <summary>
        /// Initializes a new instance of the MainViewModel class.
        /// </summary>
        public MainViewModel()
        {

            timer.Interval = TimeSpan.FromMilliseconds(500);
            timer.Tick += timer_Tick;

            LeftButtonCommand = new RelayCommand(splitOrReset);

            MainButtonCommand = new RelayCommand(()=>
                                                     {
                                                         State = State == MainState.Stopped ? MainState.Running : MainState.Stopped;
                                                     });
           
        }

        /// <summary>
        /// Unregisters this instance from the Messenger class.
        /// <para>To cleanup additional resources, override this method, clean
        /// up and then call base.Cleanup().</para>
        /// </summary>
        public override void Cleanup()
        {
            timer.Stop();
            timer.Tick -= timer_Tick;
            base.Cleanup();
        }


        /// <summary>
        /// Gets or sets the main button command.
        /// </summary>
        /// <value>The main button command.</value>
        public ICommand MainButtonCommand { get ; private set; }
        /// <summary>
        /// Gets or sets the left button command.
        /// </summary>
        /// <value>The left button command.</value>
        public ICommand LeftButtonCommand { get; private set; }

        private void splitOrReset()
        {
            if (State == MainState.Running)
            {
                SubState = SubState == SubState.Running ? SubState.Splitting : SubState.Running;
            }
            else
            {
                SubState = SubState.Running;
                startDate = DateTime.Now;
                var oldtime = ElapsedTime;
                RaisePropertyChanged("ElapsedTime",oldtime,ElapsedTime,true);
                var oldangle = SecondAngle;
                SecondAngle = 0d;
                RaisePropertyChanged("SecondAngle",oldangle,SecondAngle,true);
            }
        }

        void timer_Tick(object sender, EventArgs e)
        {
            Messenger.Default.Send(new GenericMessage<string>("TICK"));
            if (substate == SubState.Splitting) return;
            var old = SecondAngle;
            var delta = DateTime.Now - startDate;
            var ms = (delta.Seconds*1000d) + delta.Milliseconds;
            SecondAngle = (ms / 60000d) * 360d;
            RaisePropertyChanged("SecondAngle", old, SecondAngle, true);
            RaisePropertyChanged("ElapsedTime");
        }


        /// <summary>
        /// Gets or sets the second angle.
        /// </summary>
        /// <value>The second angle.</value>
        public double SecondAngle
        {
            get;
            private set;
        }


        private readonly DispatcherTimer timer = new DispatcherTimer();

        private MainState state = MainState.Stopped;
        private SubState substate = SubState.Running;
        private DateTime startDate = DateTime.Now;

        /// <summary>
        /// Gets or sets the state.
        /// </summary>
        /// <value>The state.</value>
        public MainState State
        {
            get { return state; }
            set
            {
                if (value == state) return;
                timer.Stop();
                var old = state;
                state = value;
                if (old == MainState.Stopped)
                {
                    startDate = DateTime.Now;
                    SubState = SubState.Running;
                    var oldangle = SecondAngle;
                    SecondAngle = 0d;
                    RaisePropertyChanged("SecondAngle", oldangle, SecondAngle, true);
                    timer.Start();
                }
                RaisePropertyChanged("State", old, value, true);
            }
        }

        /// <summary>
        /// Gets or sets the state of the sub.
        /// </summary>
        /// <value>The state of the sub.</value>
        public SubState SubState
        {
            get { return substate; }
            set
            {
                if (substate == value) return;
                var old = substate;
                substate = value;
                RaisePropertyChanged("SubState", old, value, true);
            }
        }

        /// <summary>
        /// Gets the elapsed time.
        /// </summary>
        /// <value>The elapsed time.</value>
        public string ElapsedTime
        {
            get
            {
                var d = (DateTime.Now - startDate);
                return (IsInDesignMode)
                           ? "01:23:59.005"
                           : string.Format("{0:00}:{1:00}:{2:00}.{3:000}",
                                           new object[] { d.TotalHours, d.Minutes, d.Seconds, d.Milliseconds });
            }
        }
    }

    /// <summary>
    /// Main State
    /// </summary>
    public enum MainState { Stopped, Running }
    /// <summary>
    /// Sub State
    /// </summary>
    public enum SubState { Splitting, Running }

}